<?php

/// Excepción estándar del controlador
/// @ingroup CoreExceptions
class ControllerException extends CoreException {}

/// La tarea asignada no existe
/// @ingroup CoreExceptions
class BadTaskException extends ControllerException {}

/// No existe el modelo seleccionado
/// @ingroup CoreExceptions
class UnexistentModelException extends ControllerException {}

/// No existe la vista seleccionada
/// @ingroup CoreExceptions
class UnexistentViewException extends ControllerException {}

/// No existe el responser seleccionado
/// @ingroup CoreExceptions
class UnexistentResponserException extends ControllerException {}

/// Fallo la redirección del sitio web
/// @ingroup CoreExceptions
class RedirectException extends ControllerException {}

/**
 * Controlador base para MoonDragon
 *
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright TuApp.net - GNU Lesser General Public License
 * @date Feb 2012
 * @version 3.2
 * @ingroup Core
 */

abstract class Controller
{
	/**
	 * Variable que verifica si llamo al constructor padre del controlador
	 * @var boolean $ready
	 */
	private $ready = false;
	
	/**
	 * Vista utilizada por el controlador
	 * @var View $view
	 */
	protected $view;

	/**
	 * Objeto para recibir parámetros POST y GET
	 * @var Request $request
	 */
	protected $request;
	
	/**
	 * Objeto responser para peticiones asíncronas
	 * @var Responser $responser
	 */
	private $responser;

	/**
	 * Constructor base del controlador
	 * @return void
	 */
	public function __construct()
	{
		if(!($this->view instanceof View))
		{
			$view = Registry::get('DEFAULT_VIEW').'View';
			if( !( class_exists( $view ) && is_subclass_of( $view, 'View' ) ) )
			{
				if( file_exists( VIEWS.'/class.'.strtolower($view).'.php' ) )
				{
					include_once VIEWS.'/class.'.strtolower($view).'.php';
				}
				else
				{
					throw new UnexistentViewException( 'No existe la vista '.$view );
				}
			}
			$this->view = new $view();
		}
		
		$this->request = new Request();
				
		Registry::set( 'controller', $this );
		
		$this->ready = true;
	}

	/**
	 * Procesa las llamadas a las tareas desconocidas
	 * @param string $name
	 * @param array $params
	 * @return void
	 * @throws BadTaskException
	 */
	public function __call( $name, $params )
	{
		throw new BadTaskException('La tarea '.$name.' no existe para el controlador '.get_class($this));
	}
	
	/**
	 * Registra la sección actual como la sección por defecto en sesión
	 * @return void
	 */
	protected function setPersistent()
	{
		Session::set('current_section', Registry::get('section'));
	}
	
	/**
	 * Traspasa el control del sistema a una subseccion
	 * @param string $subsection
	 * @return void
	 */
	protected function forwardSubsection($subsection)
	{
        $this->setPersistent();
		Session::set('current_subsection', $subsection);
		$this->doTask(Registry::get('DEFAULT_TASK'));
	}
	
	/**
	 * Regresa el control del sistema a la sección padre
	 * @return void
	 */
	protected function backToParent()
	{
		Session::set('current_subsection', false);
		$this->doTask(Registry::get('DEFAULT_TASK'));
	}
	
	/**
	 * Funcion para redireccionar a una tarea específica
	 * @param string $task
	 * @param array $params
	 * @throws ControllerException
	 * @return void
	 */
	protected function doTask($task, $params = array())
	{
		if(Registry::get('pathinfo') === true)
		{
			if(Registry::get('use_base_url') == false)
			{
				throw new ControllerException('Es necesario configurar la url base para cambiar de tarea con pathinfo');
			}
			
			$subsection = (Session::get('current_subsection') != false)?'-'.Session::get('current_subsection'):'';
			$redirect = Registry::get('base_url').'/index.php/'.Registry::get('section').$subsection.'/'.$task;
			foreach($params as  $key => $val)
			{
				$redirect .='/'.$key.'/'.$val;
			}
		}
		else
		{
			$this->setPersistent();
			$redirect = '?task='.$task;
			foreach($params as  $key => $val)
			{
				$redirect.='&'.$key.'='.$val;
			}
		}
		
		$this->redirect($redirect);
	}

	/**
	 * Registra un modelo para que sea usado en el controlador
	 * @param string $model
	 * @param string $name
	 * @return void
	 * @throws UnexistentModelException
	 */
	public function registerModel( $model, $name = '' )
	{
		if($name == '')
		{
			$name = strtolower($model);
		}
		
		$class = $model.'Model';
		
		if( !($this->$name instanceof $class) )
		{
			try{
				$this->$name = Factory::getModel($model);
			}
			catch(FactoryException $e)
			{
				throw new UnexistentModelException('No se pudo instanciar el modelo '.$model);
			}
		}
	}
	
	/**
	 * Registra la vista utilizada por el controlador
	 * @param string $view
	 * @return void
	 * @throws UnexistentViewException
	 */
	public function registerView( $view )
	{
		$class =  $view.'View';
		
		if( !($this->view instanceof $class) )
		{
			try{
				$this->view = Factory::getView($view);
			}
			catch(FactoryException $e)
			{
				throw new UnexistentViewException('No se pudo instanciar la vista '.$view);
			}
		}
	}
	
	/**
	 * Registra el responser utilizado por el controlador
	 * @param string $responser
	 * @param string $dir
	 * @return void
	 * @throws UnexistentResponserException
	 */
	public function registerResponser($responser, $dir = '')
	{
		$class = $responser.'Responser';
		if(!($this->responser instanceof $class))
		{
			try {
				$this->responser = Factory::getResponser($responser, $dir);
			}
			catch(FactoryException $e)
			{
				throw new UnexistentResponserException('No se pudo instanciar el responser '.$responser);
			}
		}
	}
	
	/**
	 * Devuelve la vista utilizada por el controlador
	 * @return View
	 */
	public function getView()
	{
		if(!$this->view instanceof View)
		{
			$view = Registry::get('DEFAULT_VIEW').'View';
			$this->view = new $view();
		}
		return $this->view;
	}

	/**
	 * Ejecuta la tarea por defecto o la tarea seleccionada
	 * @param string $task
	 * @return void
	 */
	public function run( $task = '' )
	{
		if($this->ready == false)
		{
			throw new ControllerException('No se llamo al constructor padre del controlador, error de configuración');
		}
		
		if( !$this->authenticate() )
		{
			$task = 'unauthorized';
		}
		
		if( Registry::get( 'OFFLINE_ERROR' ) && !Registry::get( 'online' ) )
		{
			$task = 'offline';
		}
		
		if( Registry::get( 'unexistent') )
		{
			$task = 'unexistent';
		}
		
		$mdview = $this->getView(); // Se asegura de que siempre haya una vista activa
		Buffer::start( 'salida' );
		
		if($task == '')
		{
			$task = Registry::get('DEFAULT_TASK');
		}
		
		if($task == 'response')
		{
			trigger_error('No se puede utilizar response como una tarea', E_USER_WARNING);
			$task = Registry::get('DEFAULT_TASK');
		}
		
		try
		{
			$this->$task();
		}
		catch(BadTaskException $e)
		{
			$this->display();
		}
		
		Buffer::end();
	}
	
	/**
	 * Obtiene una respuesta del responder seleccionado
	 * @param string $task respuesta esperada
	 * @return void
	 */
	public function response($response = '')
	{
		try{
			$view = $this->request->tryget('view', 'Simple', true);
			$this->registerView($view);
		}
		catch(UnexistentViewException $e)
		{
			$this->registerView('Simple');
		}
		
		Buffer::start( 'salida' );
		if( !$this->authenticate() )
		{
			echo 'error';
			trigger_error('No esta autorizado a acceder a este controlador', E_USER_WARNING);
			Buffer::end();
			Vars::set('success', false);
			return;
		}
		
		if( Registry::get( 'OFFLINE_ERROR' ) && !Registry::get( 'online' ) )
		{
			echo 'error';
			trigger_error('No se ha establecido una conexión con la base de datos', E_USER_WARNING);
			Buffer::end();
			Vars::set('success', false);
			return;
		}
		
		if( Registry::get( 'unexistent') )
		{
			echo 'error';
			trigger_error('No se encontró la sección especificada', E_USER_WARNING);
			Buffer::end();
			Vars::set('success', false);
			return;
		}
		
        $responser = $this->responser;
        $this->responser = new $responser();
		if($this->responser instanceof Responser)
		{
			try {
				$success = $this->responser->$response();
				Vars::set('success', $success);
			}
			catch(ResponserException $e)
			{
				echo 'error';
				Vars::set('success', false);
			}
			
		}
		else
		{
			echo 'error';
			Vars::set('success', false);
			trigger_error('No se ha asignado un responder a este controlador', E_USER_WARNING);
		}
		Buffer::end();
	}

	/**
	 * Tarea por defecto
	 * @return void
	 */
	abstract protected function display();
	
	/**
	 * Procesamiento de secciones no autorizadas
	 * @return void
	 */
	protected function unauthorized()
	{
		$view = $this->getView();
		$view->message( 'No esta autorizado a ver esta sección' ); // @todo localizar contenido
	}
	
	/**
	 * Procesamiento de secciones no existentes
	 * @return void
	 */
	protected function unexistent()
	{
		$view = $this->getView();
		$view->message( 'La sección seleccionada no existe' ); // @todo localizar contenido
	}
	
	/**
	 * Error al no existir conexión con la base de datos
	 * @return void
	 */
	protected function offline()
	{
		$view = $this->getView();
		$view->message( 'Imposible conectarse con la base de datos' ); // @todo localizar contenido		
	}

	/**
	 * Comrpueba si el sistema esta autorizado para continuar
	 * @return boolean
	 */
	protected function authenticate()
	{
		return true;
	}
	
	/**
	 * Redirige el sitio web a la dirección específicada
	 * @param string $location
	 * @return void
	 * @throws RedirectException
	 */
	protected function redirect($location)
	{
		if( Registry::get( 'stop_redirection' ) )
		{
			throw new RedirectException('Se ha deshabilitado la redirección');
		}
		else
		{
			Registry::set( 'redirect_location', $location );
		}
	}
}

//Fin de archivo
