<?php

/**
 * Gestor de errores personalizado del sistema
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright TuApp.net - GNU Lesser General Public License
 * @date Feb 2012
 * @version 3.2
 * @ingroup Core
 */
 
class ErrorHandler
{
	/**
	 * Controla si se muestran o no los errores
	 * @var boolean $report
	 */
	private static $report = true;
	
	/**
	 * Controla la salida de mensajes de depuración
	 * @var boolean $debug
	 */
	private static $debug = true;
	
	/**
	 * Controla la salida de mensajes de depuracion internos de php
	 * @var boolean $system_info
	 */
	private static $system_info = false;
	
	/**
	 * Controla la salida de mensajes de advertencia sobre fuciones deprecadas
	 * @var boolean $deprecated
	 */
	private static $deprecated = true;
	
	/**
	 * Nombre de la función utilizada para detener la ejecución del sistema
	 * @var callback $halt_callback
	 */
	private static $halt_callback = '';
	
	/**
	 * Controla si se ha registrado un error
	 * @var boolean $has_errors
	 */
	public static $has_errors = false;

   	/**
   	 * Modifica los parámetros de activación de los mensajes de depuración del sistema
   	 * @param boolean $report
   	 * @param boolean $debug
   	 * @param boolean $system_info
   	 * @param boolean $deprecated
   	 * @return void
   	 */
	public static function setDebugMode( $report = true, $debug = true, $system_info = false, $deprecated = true )
	{
		self::$report = $report;
		self::$debug = $debug;
		self::$system_info = $system_info;
		self::$deprecated = $deprecated;
	}
	
	/**
	 * Modifica la función utilizada para detener la ejecución del sistema
	 * @param callback $function_name funcion( $error_string, $debug, $error_file, $error_line, $error_trace ) 
	 * @return void
	 */	
	public static function setHaltHandler( $function_name )
	{
		self::$halt_callback = $function_name;
	}

	/**
	 * Manejador de error personalizado
	 * @param int $error_number
	 * @param string $error_string
	 * @param string $error_file
	 * @param int $error_line
	 * @param array $error_context
	 * @return boolean
	 */
	public static function handler( $error_number, $error_string, $error_file, $error_line, $error_context )
	{
		//Si se ha desactivado la generación de errores se sale de la función
		if( error_reporting() == 0 )
		{
			return true;
		}
		
		switch( $error_number )
		{
			case E_USER_ERROR:
				
				if(is_writable(LOGS))
				{
					$error_log = date( 'd/m/Y H:i:s' )."\n";// @todo localizar contenido
					$error_log .= 'Error Fatal: "'.$error_string.'" en '.$error_file.' línea '.$error_line."\n";// @todo localizar contenido
					$error_log .= "Contexto:\n".print_r( $error_context, true )."\n\n";// @todo localizar contenido				
					file_put_contents( LOGS.'/error.log', $error_log, FILE_APPEND | LOCK_EX );
				}
				
				$halt_callback = self::$halt_callback;
				if( function_exists( $halt_callback ) )
				{
					$halt_callback( $error_string, self::$report, $error_file, $error_line, '');
				}
				else
				{
					die( 'Error Fatal: "'.$error_string.'" en '.$error_file.' línea '.$error_line.'. Consulte el log de errores para obtener más detalles.' ); //@todo localizar contenido
				}
				break;
			case E_WARNING:
			case E_USER_WARNING:
				if( !self::$report )
				{
					return true;
				}
				
				self::$has_errors = true;
				
				Buffer::write( "e_warning", 'Advertencia: "'.$error_string.'" en '.$error_file.' línea '.$error_line."\n" );// @todo localizar contenido
				
				break;
			case E_USER_NOTICE:
				if( !self::$debug || !self::$report)
				{
					return true;
				}
				
				self::$has_errors = true;
				
				Buffer::write( "e_notice", 'Mensaje: "'.$error_string.'" en '.$error_file.' línea '.$error_line."\n" );// @todo localizar contenido
								
				break;
			case E_NOTICE:
			case E_STRICT:
				if( !self::$system_info || !self::$report )
				{
					return true;
				}
				
				self::$has_errors = true;
				
				Buffer::write( "e_notice", 'Mensaje: "'.$error_string.'" en '.$error_file.' línea '.$error_line."\n" );// @todo localizar contenido
								
				break;
			case E_DEPRECATED:
			case E_USER_DEPRECATED:
				if( !self::$deprecated || !self::$report )
				{
					return true;
				}
				
				self::$has_errors = true;
				
				Buffer::write( "e_notice", 'Mensaje: "'.$error_string.'" en '.$error_file.' línea '.$error_line."\n" );// @todo Localizar contenido
								
				break;
			default:
				//error desconocido, se pasa el control al gestor por defecto de php
				return false;
		}
				
		return true;
	}
	
	/**
	 * Manejador de excepciones personalizado
	 * @param Exception $exception
	 * @return void
	 */
	public static function exhandler($exception)
	{
		if(is_writable(LOGS))
		{
			$error_log = date( 'd/m/Y H:i:s' )."\n";// @todo localizar contenido
			$error_log .= 'Excepcion no capturada: "'.$exception->getMessage().'" en '.$exception->getFile().' línea '.$exception->getLine()."\n";// @todo localizar contenido
			$error_log .= "Contexto:\n".$exception->getTraceAsString()."\n\n";// @todo localizar contenido
			file_put_contents( LOGS.'/error.log', $error_log, FILE_APPEND | LOCK_EX );
		}
		
		$halt_callback = self::$halt_callback;
		if( function_exists( $halt_callback ) )
		{
			$halt_callback( 'Excepción no capturada '.get_class($exception).': '.$exception->getMessage(), self::$report, $exception->getFile(), $exception->getLine(), $exception->getTraceAsString() );
		}
		else
		{
			die( 'Error Fatal: "'.$exception->getMessage().'" en '.$exception->getFile().' línea '.$exception->getLine().'. Consulte el log de errores para obtener más detalles.' ); //@todo localizar contenido
		}
	}
	
	/**
	 * Inicializa el gestor de errores
	 * @param boolean $report
	 * @param boolean $debug
	 * @param boolean $system_info
	 * @param boolean $deprecated
	 * @param callback $halt_callback function( $error_string, $error_file, $error_line )
	 * @return void
	 */
	public static function init( $report = true, $debug = true, $system_info = false, $deprecated = true, $halt_callback = '' )
	{
		set_error_handler( array( 'ErrorHandler', 'handler' ) );
		set_exception_handler( array( 'ErrorHandler', 'exhandler' ) );
        self::setDebugMode($report, $debug, $system_info, $deprecated);
		self::setHaltHandler( $halt_callback );
	}
	
	/**
	 * Obtiene los mensajes de advertencia del sistema
	 * @return string
	 */
	public static function getWarnings()
	{
		return Buffer::getContent('e_warning', false);
	}
	
	/**
	 * Obtiene los mensajes de notificación del sistema
	 * @return string
	 */
	public static function getNotices()
	{
		return Buffer::getContent('e_notice', false);
	}
}
 
// Fin de archivo
