<?php

/**
 * Vista base para salida en HTML
 *
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright TuApp.net - GNU Lesser General Public License
 * @date Feb 2012
 * @version 3.2
 * @ingroup Core
 */

abstract class HTMLView extends View
{
	/**
	 * Página principal de la aplicación
	 * @var string $page
	 */
	protected $page;

	/**
	 * Plantillas utilizadas por la vista
	 * @var array $templates
	 */
	protected $templates;

	/**
	 * Contenido principal de la página
	 * @var string $content
	 */
	protected $content;

	/**
	 * Controla si se ha seleccionado una página manualmente
	 * @var boolean $custom_page
	 */
	protected $custom_page = false;

	/**
	 * Constructor de la clase
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();
		$this->page = Registry::get( 'DEFAULT_PAGE' );
		$this->templates = array();
	}

	/**
	 * Agrega contenido a la vista
	 * @param string $content
	 * @return void
	 */
	public function addContent( $content )
	{
		$this->content .= $content;
	}

	/**
	 * Modifica la página principal de la vista
	 * @param string $page
	 * @return void
	 */
	public function setPage( $page )
	{
		$this->custom_page = true;
		$this->page = $page;
	}
	
	/**
	 * Override del mensaje de error de la vista
	 * @return string
	 * @see View::getErrors()
	 */
	public function getErrors()
	{
		$errors = '<pre class="info_error">';
		$errors .= parent::getErrors();
		$errors .= '</pre>';
		
		return $errors;
	}
	
	/**
	 * Inserta un script externo en la plantilla html
	 * @param string $src
	 * @param boolean $page_end determina si el script va al final de la página
	 * @return void
	 */
	public function addScript($src, $page_end = false)
	{
		$script = '<script type="text/javascript" src="'.$src.'"></script>'."\n";
		if($page_end == true)
		{
			Buffer::write('script_tags', $script);
		}
		else
		{
			Buffer::write('meta_header', $script);
		}
	}
	
	/**
	 * Inserta una hoja de estilos externa en la plantilla html
	 * @param string $src
	 * @return void
	 */
	public function addStyle($src)
	{
		$style = '<link href="'.$src.'" rel="stylesheet" type="text/css" />';
		Buffer::write('meta_header', $style);
	}
	
	/**
	 * Inserta un metatag en la plantilla html
	 * @param string $meta
	 * @return void
	 */
	public function addMeta($meta)
	{
		Buffer::write('meta_header', $meta);
	}

	/**
	 * Override del método show para mostrar el contenido
	 * @return void
	 * @see View::show()
	 */
	public function show()
	{
		$message = Buffer::getContent( 'message' );

		if( $message != '' )
		{
			$this->addVar( 'message', $message );
		}

		try
		{
			$page = new Page( $this->page );
			$page->setVars( $this->getVars() );
			$page->setMeta(Buffer::getContent('meta_header'));
			$page->setScript(Buffer::getContent('script_tags'));

			try
			{
				foreach( $this->templates as $name => $template )
				{
					$tpl = Template::load( $template, $this->vars );
						
					$page->addTemplate( $name, $tpl );
				}
			}
			catch(TemplateException $e)
			{
				throw new ViewException('Ocurrió un error con las plantillas definidas en la vista');
			}

			$content =  $this->getOutput();

			if( ErrorHandler::$has_errors )
			{
				$error =  $this->getErrors();
			}
			else
			{
				$error = '';
			}
			
			$page->setError($error);
			$page->setContent( $content.$this->content );

			return $page->show();
		}
		catch(PageException $e)
		{
			throw new ViewException('Ocurrió un error con la página definida en la vista');
		}
	}

	/**
	 * Override del método message para publicar un mensaje
	 * @param string $string
	 * @return void
	 * @see View::message()
	 */
	public function message( $string )
	{
		$this->setPage( Registry::get( 'MESSAGE_PAGE' ));
		Buffer::write( 'message', $string, true );
	}

	/**
	 * Agrega una plantilla a la vista
	 * @param string $name
	 * @param string $template
	 * @return void
	 */
	public function addTemplate( $name, $template )
	{
		$this->templates[$name] = $template;
	}
}

// Fin de archivo
