<?php

/// Errores en la carga de librerías
/// @ingroup CoreExceptions
class LibraryException extends CoreException {}

/**
 * Se encarga de cargar las librerias
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright TuApp.net - GNU Lesser General Public License
 * @date Feb 2012
 * @version 3.2
 * @ingroup Core
 */
 
 class Libs
{	
	/**
	 * Listado de librerías
	 * @var array $libraries
	 */
	private static $libraries;

	/**
	 * Comprueba si la librería es compatible con el sistema
	 * @param Lib $libobject
	 * @return void
	 * @throws LibraryException
	 */
	private static function checkCompatibility( Lib $libobject )
	{
		if($libobject->name == 'core')
		{
			return;
		}
		
		$md_version = self::getVersion( Registry::get( 'MOONDRAGON_VERSION' ) );
		$lib_version = self::getVersion( $libobject->moondragon_version );
		
		if(!self::checkVersion( $md_version, $lib_version ))
		{
			throw new LibraryException( 'La librería '.$library.' no es compatible con esta instalación de MoonDragon' ); // @todo localizar contenido
		}
	}
	
	/**
	 * Comprueba si las dependencias de la librería se satisfacen
	 * @param Lib $libobject
	 * @return void
	 * @throws LibraryException
	 */
	private static function checkDependencies( Lib $libobject )
	{
		$dependencies = $libobject->getDependencies();
		
		foreach( $dependencies as $lib => $version )
		{
			if( !isset( self::$libraries[strtolower($lib)] ) )
			{
				self::import( $lib  );
			}
			
			if( !self::checkVersion( self::getVersion( self::$libraries[strtolower($lib)]->version ), self::getVersion( $version ) ) )
			{
				throw new LibraryException( 'La versión de '.$lib.' es incompatible' ); // @todo localizar contenido
			}
		}		
	}
	
	/**
	 * Desglosa los números de versión en un arreglo a partir de una cadena
	 * @param string $version
	 * @return array
	 */	
	private static function getVersion( $version )
	{
		list( $ver ) = explode( ' ', $version );
		
		$ver_array = explode( '.', $ver );
		
		$mayor = isset($ver_array[0])?$ver_array[0]:0;
		$minor = isset($ver_array[1])?$ver_array[1]:0;
		$bugfixes = isset($ver_array[2])?$ver_array[2]:0;
		
		return array( 'mayor' => $mayor, 'minor' => $minor, 'bugfixes' => $bugfixes );
	}
	
	/**
	 * Comprueba la compatibilidad entre dos versiones
	 * @param array $actual_version
	 * @param array $required_version
	 * @return boolean
	 */	
	private static function checkVersion( $actual_version, $required_version )
	{
		if( $actual_version['mayor'] == $required_version['mayor'] )
		{
			if( $actual_version['minor'] == 'x' || $actual_version['minor'] >= $required_version['minor'] )
			{
				return true;
			}
		}
		
		return false;
	}
		
	/**
	 * Agrega una librería al sistema
	 * @param string $library
	 * @param boolean $full Determina si es una importación completa, por defecto es falso
	 * @return void
	 * @throws LibrayException
	 */
	public static function import( $library, $full = false )
	{
		if( isset( self::$libraries[strtolower($library)] ) )
		{
			return;
		}
		
		$file = LIB.'/'.strtolower($library).'/lib.'.strtolower($library).'.xml';
		
		if(!file_exists($file))
		{
			throw new LibraryException( 'No se ha podido incluir la librería '.$library); // @todo localizar contenido
		}
		
		$xml = simplexml_load_file( $file );
		
		$name = $xml['name'];
		$version = $xml['version'];
		$moondragon_version = $xml['moondragon-version'];
		
		$libobject = new Lib($name, $version, $moondragon_version);
		
		if(isset($xml->dependencies))
		{
			foreach($xml->dependencies->lib as $deplib)
			{
				$libobject->addDependency( (string) $deplib,  (string) $deplib['version']);
			}
		}
		
		self::checkCompatibility( $libobject );
		self::checkDependencies( $libobject );
		
		if(isset($xml->constants))
		{
			foreach($xml->constants->var as $var)
			{
				define((string) $var['name'], (string) $var);
			}
		}
		
		if(isset($xml->classes))
		{
			foreach($xml->classes->class as $class)
			{
				if(class_exists($class, false) && $library != 'Core')
				{
					throw new LibraryException('La clase '.$class.' de la librería '.$library.' esta en conflicto con otra clase existente');
				}
				
				if($class['essential'] == 'true' || $full === true)
				{
					require_once LIB.'/'.strtolower($library).'/classes/class.'.$class.'.php';
				}
			}
		}
		
		if(isset($xml->functions))
		{
			foreach($xml->functions->function as $function)
			{
				require_once LIB.'/'.strtolower($library).'/functions/function.'.$function.'.php';
			}
		}

		if(isset($xml->config))
		{
			foreach($xml->config->conf as $conf)
			{
				require_once LIB.'/'.strtolower($library).'/conf.'.$conf.'.inc';
			}
		}
		
		self::$libraries[strtolower($library)] = $libobject;
	}
	
	/**
	 * Obtiene la lista de librerías cargadas
	 * @return array
	 */
	public static function getLibraries()
	{
		$libs = array();
		foreach(self::$libraries as $key => $lib)
		{
			$libs[] = $key;
		}
		
		return $libs;
	}
}
 
// Fin de archivo
