<?php

/// Error en el procesamiento de la plantilla de página
/// @ingroup CoreExceptions
class PageException extends TemplateException {}

/**
 * Controla la publicación de una página web
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright TuApp.net - GNU Lesser General Public License
 * @date Feb 2012
 * @version 3.2
 * @ingroup Core
 */
 
class Page
{
	/**
	 * Nombre de la página
	 * @var string $page
	 */
	private $page;
	
	/**
	 * Plantillas añadidas a la página
	 * @var array $templates
	 */
	private $templates;
	
	/**
	 * Contenido principal de la página
	 * @var string $content
	 */
	private $content;
	
	/**
	 * Etiquetas de metatags y css publicadas en la sección head
	 * @var string $meta
	 */
	private $meta;
	
	/**
	 * Etiquetas de script que van al final de la página
	 * @var string $script
	 */
	private $script;
	
	/**
	 * Etiqueta de error del sistema
	 * @var string $error
	 */
	private $error;
	
	/**
	 * Variables de contenido
	 * @var array $vars
	 */
	private $vars = array();

	/**
	 * Constructor de la clase
	 * @param string $page
	 * @return void
	 */
	public function __construct( $page )
	{
		$this->page = $page;
		$this->content = '';
		$this->meta = '';
	}
	
	/**
	 * Método mágico para convertir la clase en una cadena
	 * @return string
	 */
	public function __toString()
	{
		return $this->show();
	}

	/**
	 * Agrega una plantilla a la página
	 * @param string $name
	 * @param string $template
	 * @return void
	 */
	public function addTemplate( $name, $template )
	{
		$this->templates[$name] = $template;
	}
	
	/**
	 * Modifica el contenido principal de la página
	 * @param string $content
	 * @return void
	 */
	public function setContent( $content )
	{
		$this->content = $content;
	}
	
	
	/**
	 * Modifica las etiquetas de la sección head
	 * @param string $meta
	 * @return void
	 */
	public function setMeta($meta)
	{
		$this->meta = $meta;
	}
	
	/**
	 * Modifica las etiquetas de scripts de la página
	 * @param string $script
	 * @return void
	 */
	public function setScript($script)
	{
		$this->script = $script;
	}
	
	/**
	 * Modifica el mensaje de error del sistema
	 * @param string $error
	 * @return void
	 */
	public function setError($error)
	{
		$this->error = $error;
	}
	
	/**
	 * Modifica las variables de contenido
	 * @param array $vars
	 * @return void
	 */
	public function setVars( $vars )
	{
		$this->vars = $vars;
	}

	/**
	 * Muestra el contenido de la página
	 * @return string
	 */
	public function show()
	{
		$content = $this->process();
		
		return $content;
	}

	/**
	 * Procesa una página para agregar los fragmentos indicados
	 * @return string
	 * @throws PageException
	 */
	private function process()
	{
		try
		{
 			$template = new Template( $this->page, true );
			$template->setVars( $this->vars );
		
			$htmlText = $template->show();
		}
		catch(TemplateException $e)
		{
			throw new PageException( 'Error con la plantilla de la página '.$this->page );
		}
		
		preg_match_all('/<md:(\w*|\w+:\w+)\/>/i', $htmlText, $templates);
				
		$htmlText = str_replace('<md:content/>', $this->content, $htmlText);
		foreach($templates[1] as $tpl)
		{
			// $texts[$tpl] = $this->loadTemplate($tpl);	
			$htmlText = str_replace('<md:'.$tpl.'/>', $this->loadTemplate($tpl), $htmlText);
		}
		
		$htmlText = str_replace('</head>', $this->meta.'</head>', $htmlText);
		$htmlText = str_replace('</body>', $this->error.'</body>', $htmlText);
		$htmlText = str_replace('</body>', $this->script.'</body>', $htmlText);
		
		if(Registry::get('use_base_url') === true)
		{
			$htmlText = str_replace('<head>', '<head><base href="'.Registry::get('base_url').'/index.php" />', $htmlText);
		}

		return $htmlText;
	}
	
	/**
	 * Carga la plantilla seleccionada
	 * @param string $template
	 * @return string
	 */
	private function loadTemplate( $template )
	{
		if( !isset( $this->templates[$template] ) )
		{
			try
			{
				$tpl = new Template( $template );
				$tpl->setVars( $this->vars );
				$tplstr = $tpl->show();
				$this->addTemplate($template, $tplstr);
			}
			catch(TemplateException $e)
			{
				return '';
			}
		}
		
		return $this->templates[$template];
	}
}
 
// Fin de archivo
