<?php

/// Error durante el procesamiento de variables POST y GET
/// @ingroup CoreExceptions
class RequestException extends CoreException{}

/**
 * Filtro contra XSS de variables POST y GET
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright TuApp.net - GNU Lesser General Public License
 * @date Feb 2012
 * @version 3.2
 * @ingroup Core
 */

class Request
{
	/**
	 * Devuelve la variable POST indicada
	 * @param string $id
	 * @param boolean $filter determina si el resultado se filtra contra XSS, por defecto falso
	 * @return string
	 * @throws RequestException
	 */
	public function post($id, $filter = false)
	{
		if(!isset($_POST[$id]))
		{
			throw new RequestException('No se ha recibido la variable '.$id.' en POST');
		}
		
		$value = $_POST[$id];
		if($filter == true)
		{
			$value = $this->filterXSS($value);
		}
		
		return $value;
	}
	
	/**
	 * Devuelve la variable POST indicada o la variable alternativa si no se existe
	 * @param string $id
	 * @param string $alt variable alternativa
	 * @param boolean $filter determina si el resultado se filtra contra XSS, por defecto falso
	 * @return string
	 * @throws RequestException
	 */
	public function trypost($id, $alt, $filter = false)
	{
		if(!isset($_POST[$id]))
		{
			return $alt;
		}
		
		$value = $_POST[$id];
		if($filter == true)
		{
			$value = $this->filterXSS($value);
		}
		
		return $value;
	}
	
	/**
	 * Devuelve la variable GET indicada o la variable de PATH_INFO
	 * @param string $id
	 * @param boolean $filter determina si el resultado se filtra contra XSS, por defecto falso
	 * @return string
	 * @throws RequestException
	 */
	public function get($id, $filter = false)
	{
		if(isset($_GET[$id]))
		{
			$value = $_GET[$id];
		}
		else
		{
			if(PathVars::hasVars() == true)
			{
				try
				{
					$value = PathVars::get($id);
				}
				catch(PathVarsException $e)
				{
					throw new RequestException('No se ha recibido la variable '.$id.' en GET');
				}
			}
			else
			{
				throw new RequestException('No se ha recibido la variable '.$id.' en GET');
			}
		}	
		
		if($filter == true)
		{
			$value = $this->filterXSS($value);
		}
		
		return $value;
	}
	
	/**
	 * Devuelve la variable GET indicada o la variable de PATH_INFO, o la variable alternativa si no existe
	 * @param string $id
	 * @param string $alt variable alternativa
	 * @param boolean $filter determina si el resultado se filtra contra XSS, por defecto falso
	 * @return string
	 * @throws RequestException
	 */
	public function tryget($id, $alt, $filter = false)
	{
		if(isset($_GET[$id]))
		{
			$value = $_GET[$id];
		}
		else
		{
			if(PathVars::hasVars() == true)
			{
				try
				{
					$value = PathVars::get($id);
				}
				catch(PathVarsException $e)
				{
					return $alt;
				}
			}
			else
			{
				return $alt;
			}
		}	
		
		if($filter == true)
		{
			$value = $this->filterXSS($value);
		}
		
		return $value;
	}
	
	/**
	 * Devuelve la variable que sea de POST, GET o PATH_INFO
	 * @param string $id
	 * @param boolean $filter determina si el resultado se filtra contra XSS, por defecto falso
	 * @return string
	 * @throws RequestException
	 */
	public function all($id, $filter = false)
	{
		if(isset($_REQUEST[$id]))
		{
			$value = $_REQUEST[$id];
		}
		else
		{
			if(PathVars::hasVars() == true)
			{
				try
				{
					$value = PathVars::get($id);
				}
				catch(PathVarsException $e)
				{
					throw new RequestException('No se ha recibido la variable '.$id.' en POST ni GET');
				}
			}
			else
			{
				throw new RequestException('No se ha recibido la variable '.$id.' en POST ni GET');
			}
		}
		
		if($filter == true)
		{
			$value = $this->filterXSS($value);
		}
		
		return $value;
	}
	
	/**
	 * Devuelve la variable que sea de POST, GET o PATH_INFO, o la variable alternativa si no existe
	 * @param string $id
	 * @param string $alt variable alternativa
	 * @param boolean $filter determina si el resultado se filtra contra XSS, por defecto falso
	 * @return string
	 * @throws RequestException
	 */
	public function tryall($id, $alt, $filter = false)
	{
		if(isset($_REQUEST[$id]))
		{
			$value = $_REQUEST[$id];
		}
		else
		{
			if(PathVars::hasVars() == true)
			{
				try
				{
					$value = PathVars::get($id);
				}
				catch(PathVarsException $e)
				{
					return $alt;
				}
			}
			else
			{
				return $alt;
			}
		}
		
		if($filter == true)
		{
			$value = $this->filterXSS($value);
		}
		
		return $value;
	}
	
	/**
	 * Filtra la variable recibida contra ataques XSS
	 * @param string $data
	 * @return string
	 */
	private function filterXSS($data)
	{
		$data = str_replace('&', '&amp;', $data);
		$data = str_replace('#', '&#35;', $data);
		$data = str_replace('<', '&lt;', $data);
		$data = str_replace('>', '&gt;', $data);
		$data = str_replace('(', '&#40;', $data);
		$data = str_replace(')', '&#41;', $data);
		$data = str_replace('"', '&quot;', $data);
		$data = str_replace("'", '&#39;', $data);
		
		return $data;
	}
}

// Fin
