<?php

/// Error en el procesamiento de la plantilla
/// @ingroup CoreExceptions
class TemplateException extends CoreException {}

/**
 * Manejador de plantillas
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright TuApp.net - GNU Lesser General Public License
 * @date Feb 2012
 * @version 3.2
 * @ingroup Core
 */
 
class Template
{
	/**
	 * Nombre de la librería que contiene la plantilla
	 * @var string $lib
	 */
	private $lib = '';
	
	/**
	 * Nombre de la plantilla actual
	 * @var string $template
	 */
	private $template;
			
	/**
	 * Variables para ser reemplazadas en la plantilla
	 * @var array $vars
	 */
	private $vars = array();
	
	/**
	 * Contenido de la plantilla
	 * @var string $content
	 */
	private $content = '';
	
	/**
	 * Constructor de la clase
	 * @param string $template
	 * @param boolean $page Determina si la plantilla es una página, por defecto es falso 
	 * @return void
	 */
	public function __construct( $template = '', $page = false )
	{
		$this->setTemplate( $template, $page );
	}
	
	/**
	 * Método mágico para convertir el objeto en una cadena
	 * @return string
	 */
	public function __toString()
	{
		return $this->show();
	}

	/**
	 * Modifica la plantilla seleccionada
	 * @param string $template
	 * @param boolean $page Determina si la plantilla es una página, por defecto es falsa
	 * @return void
	 */
	public function setTemplate( $template, $page = false )
	{
		if( strpos( $template, ':' ) !== false )
		{
			list( $lib, $template ) = explode( ':', $template );
			$this->lib = $lib;
		}
		$this->template =  ($page == true )?'page.'.$template:$template;
		$this->content = '';
	}
	
	/**
	 * Modifica las variables que serán agregadas a la plantilla
	 * @param array $vars
	 * @return void
	 */
	public function setVars( $vars )
	{
		$this->vars = $vars;
	}

	/**
	 * Devuelve el contenido de la plantilla despues de procesarlo
	 * @return string
	 * @throws TemplateException
	 */
	public function show()
	{
		if( $this->content == '' )
		{
			$this->readTemplate();
			$this->replaceVars();
		}
		
		return $this->content;
	}
	
	/**
	 * Lee el archivo de plantilla
	 * @return void
	 * @throws TemplateException
	 */
	private function readTemplate()
	{
		if($this->lib != '')
		{
			$file = LIB.'/'.$this->lib.'/templates/'.$this->template.'.tpl';
		}
		else 
		{
			$file =  TPL.'/'.$this->template.'.tpl';
		}
		
		if( !file_exists( $file ) )
		{
			throw new TemplateException( 'La plantilla '.$file.' no existe' );//@todo localizar contenido
		}
		
		$file_data = @fopen($file, 'r');
		if ( !$file_data ) 
		{
			$this->content = '';
			throw new TemplateException( 'No se pudo leer la plantilla' );//@todo localizar contenido
		}
		
		$this->content = fread( $file_data, filesize( $file ) );
		fclose( $file_data );
	}
	
	/**
	 * Reemplaza las variables en la plantilla
	 * @return void
	 */
	private function replaceVars()
	{			
		foreach($this->vars as $key => $var)
		{
			$this->content = str_replace( '[#:'.$key.']', $var, $this->content );
		}
		$this->content = preg_replace( '/\[\#:(\w*)\]/s', '', $this->content );
	}

	/**
	 * Procesa y devuelve la plantilla específicada
	 * @param string $template
	 * @param array $vars
	 * @param boolean $page Determina si la plantilla es una página, por defecto es falso
	 * @return string
	 * @throws TemplateException
	 */	
	public static function load( $template, $vars = array(), $page = false )
	{
		$tpl = new self( $template, $page );
		$vars_array = array_merge(  Vars::getVars(), $vars );
		$tpl->setVars( $vars_array );
		return $tpl->show();
	}
}
 
// Fin de archivo
