<?php

/**
 * Controlador base para el manejo de usuarios
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright TuApp.net - GNU Lesser General Public License
 * @date May 2012
 * @version 1.0
 * @ingroup UserManager
 */
abstract class UmController extends Controller
{
	/**
	 * Datos de configuración del usuario que estan siempre disponibles
	 * @var object $userdata
	 */
	protected $userdata;
	
	/**
	 * Plantilla de formulario de login
	 * @var string $login_form
	 */
	protected $login_form = 'usermanager:login';

	/**
	 * Página para el formulario de login
	 * @var string $login_page
	 */
    protected $login_page = 'standard';
	
    /**
     * Grupos autorizados a entrar en el controlador
     * @var array $allowed_groups
     */
	protected $allowed_groups = array();
	
	/**
	 * Construccion y configuracion del controlador
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();
		$this->registerModel('Um');
		
		$this->userdata = new StdClass();
		$this->userdata->id = 0;
		$this->userdata->usuario = 'anonimo';
		$this->userdata->email = '';
		$this->userdata->nombres = '';
		$this->userdata->apellidos = '';
	}
	
	/**
	 * Muestra el formulario de login
	 * @return void
	 */
	public function login_form()
	{
		$this->view->setPage($this->login_page);
		if(Session::get('usuario_conectado') === true)
		{
			$this->doTask('display');
		}
		else
		{
			$message = Session::get('login_message');
			if($message)
			{
				Vars::set('login_message', $message);
				Session::set('login_message', false);
			}
			echo Template::load($this->login_form);
		}
	}
	
	/**
	 * Proceso de ingreso del usuario
	 * @return void
	 */
	public function login()
	{
		if(Session::get('usuario_conectado') === true)
		{
			$this->doTask('display');
		}
		else
		{
			try {
				$usuario = $this->request->post('usuario', true);
				$password = $this->request->post('contrasena', true);
				
				if($this->um->login($usuario, $password) == true)
				{
					Session::set('usuario_conectado', true);
					$id = $this->um->getIdUsuario($usuario);
					Session::set('id_usuario', $id);
					$this->fillData($id);
					$this->doTask('display');
				}
				else
				{
					Session::set('usuario_conectado', false);
					Session::set('id_usuario', false);
					Session::set('login_message', 'Usuario o contraseña incorrectos');
					$this->doTask('login_form');
				}			
			}
			catch(RequestException $e)
			{
				Session::set('usuario_conectado', false);
				Session::set('id_usuario', false);
				Session::set('login_message', 'No se recibió usuario y contraseña');
				$this->doTask('login_form');
			}
			catch(QueryException $e)
			{
				Session::set('usuario_conectado', false);
				Session::set('id_usuario', false);
				Session::set('login_message', 'Ocurrió un error en la base de datos');
				$this->doTask('login_form');
			}
		}
	}
	
	/**
	 * Proceso de logout del usuario
	 * @return void
	 */
	public function logout()
	{
		Session::set('usuario_conectado', false);
		Session::set('id_usuario', false);
		
		$_SESSION = array();
		
		$this->doTask('login_form');
	}
	
	/**
	 * Método que autentica si el usuario es válido para utilizar el controlador
	 * @return void
	 * @see Controller::authenticate()
	 */
	protected function authenticate()
	{
		if(Registry::get('task') == 'login' || Registry::get('task') == 'login_form')
		{
			return true;
		}
		
		if(Session::get('usuario_conectado') === true)
		{
			$id = Session::get('id_usuario');
			try {
				$this->fillData($id);
			}
			catch(QueryException $e)
			{
				// Algo esta mal con el usuario en sesion
				return false;
			}
			
			if(empty($this->allowed_groups) || $this->userdata->administrador == 1) // Se asume que todos son permitidos o administrador
			{
				return true;
			}
			else
			{
				foreach($this->allowed_groups as $group)
				{
					if($this->checkGroup($group) == true)
					{
						return true;
					}
				}
				
				return false;
			}
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Si el usuario no esta autorizado lo envia al formulario de login
	 * @return void
	 * @see Controller::unauthorized()
	 */
	protected function unauthorized()
	{
		$this->view->setPage($this->login_page);
		if(Session::get('usuario_conectado') == false)
		{
			//Vars::set('login_message', 'No esta autorizado a ver esta sección');
			echo Template::load($this->login_form);
		}
		else
		{
			echo '<p class="mensaje">No tiene permiso de ver esta sección<br/><a href="index.php">Ir al inicio</a></p>';
		}
	}
	
	/**
	 * Carga la información del usuario desde la base de datos a un objeto accesible por el controlador
	 * @param int $id
	 * @return void
	 */
	protected function fillData($id)
	{
		$data = $this->um->get($id);
		
		$this->userdata->id = $id;
		$this->userdata->usuario = $data->usuario;
		$this->userdata->email = $data->email;
		$this->userdata->nombres = $data->nombres;
		$this->userdata->apellidos = $data->apellidos;
		$this->userdata->administrador = $data->administrador;
		$this->userdata->grupos = $data->grupos;
	}
	
	/**
	 * Verifica si el usuario activo pertenece al grupo especificado
	 * @param string $group
	 * @return boolean
	 */
	protected function checkGroup($group)
	{
		foreach($this->userdata->grupos as $grupo)
		{
			if($grupo->nombre_grupo == $group)
			{
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * Agrega un grupo autorizado al controlador, si no hay grupos autorizados se asume que son todos
	 * @param string $group
	 * @return void
	 */
	protected function addGroup($group)
	{
		$this->allowed_groups[] = $group;
	}
}

// Fin de archivo
